<?php

namespace Drupal\harmonize\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\harmonize\Constants\HarmonizeConfig;

/**
 * Provides a configuration form for the Harmonizer module.
 *
 * @package Drupal\harmonize\Form
 */
class HarmonizeSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() : string {
    return 'harmonizer_admin_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() : array {
    return [
      HarmonizeConfig::MAIN_CONFIG_NAME,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) : array {

    // Set this for better organization of form values later.
    $form['#tree'] = TRUE;

    // Get the current configurations.
    $auto_pre_processor = $this->config(HarmonizeConfig::MAIN_CONFIG_NAME)->get(HarmonizeConfig::AUTO_PRE_PROCESSOR);
    $harmony = $this->config(HarmonizeConfig::MAIN_CONFIG_NAME)->get(HarmonizeConfig::HARMONY);
    $remove_field_underscore_prefix = $this->config(HarmonizeConfig::MAIN_CONFIG_NAME)->get(HarmonizeConfig::REMOVE_FIELD_UNDERSCORE_PREFIX);
    $max_recursive_depth = $this->config(HarmonizeConfig::MAIN_CONFIG_NAME)->get(HarmonizeConfig::MAX_RECURSIVE_DEPTH);

    // Enable the Auto-Preprocessor.
    $form[HarmonizeConfig::AUTO_PRE_PROCESSOR] = [
      '#type'          => 'checkbox',
      '#title'         => t('Enable the Auto-Pre-Processor for Templates'),
      '#description'   => t(
        '<em>Check this box to automatically preprocess entities and objects. This will add a <strong>data</strong> variable to most templates, including harmonized data.</em>'
      ),
      '#default_value' => !empty($auto_pre_processor) ? $auto_pre_processor : 0,
      '#weight'        => -15,
    ];

    // Enable the automatic removal "field_" prefix when harmonization fields.
    $form[HarmonizeConfig::REMOVE_FIELD_UNDERSCORE_PREFIX] = [
      '#type'          => 'checkbox',
      '#title'         => t('Remove <strong>"field_"</strong> prefix when harmonizing Entity Fields.'),
      '#description'   => t(
        '<em>Check this box to remove the prefix from field names when harmonizing entity fields. Array keys for specific fields will have the full field name otherwise.'
      ),
      '#default_value' => !empty($remove_field_underscore_prefix) ? $remove_field_underscore_prefix : 0,
      '#weight'        => -15,
    ];

    // Enable the automatic removal "field_" prefix when harmonization fields.
    $form[HarmonizeConfig::MAX_RECURSIVE_DEPTH] = [
      '#type'          => 'select',
      '#title'         => t('Select the max depth for recursive harmonization in Entities.'),
      '#description'   => t(
        '<em>A failsafe is developed to prevent infinite loops. After reaching a certain depth, the data will no longer be automatically pre-processed. Instead, the data will be replaced by the harmonizer class itself. You can then call the getData() function manually on that class to obtain the data desired. You can customize how low the module goes by default.'
      ),
      '#options'       => [
        0 => 0,
        1 => 1,
        2 => 2,
        3 => 3,
        4 => 4,
        5 => 5,
        6 => 6,
      ],
      '#default_value' => !empty($max_recursive_depth) ? $max_recursive_depth : 3,
      '#weight'        => -15,
    ];

    // Enable the use of Harmony.
    $form[HarmonizeConfig::HARMONY] = [
      '#type'          => 'checkbox',
      '#title'         => t('Enable Harmony for your Custom Theme.'),
      '#description'   => t(
        '<em>Check this box to enable Harmony. After enabling this, copy the <strong>harmony</strong> folder found in the ./tools folder of the module to the <strong>root of your custom theme</strong>. Please refer to the README.md documentation for instructions!'
      ),
      '#default_value' => !empty($harmony) ? $harmony : 0,
      '#weight'        => -15,
    ];

    // Return the form with all necessary fields.
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) : void {
    // Retrieve the configuration.
    $values = $form_state->getValues();

    unset($values['actions']);

    $this->configFactory->getEditable(HarmonizeConfig::MAIN_CONFIG_NAME)
      ->set(HarmonizeConfig::AUTO_PRE_PROCESSOR, $values[HarmonizeConfig::AUTO_PRE_PROCESSOR])
      ->set(HarmonizeConfig::HARMONY, $values[HarmonizeConfig::HARMONY])
      ->set(HarmonizeConfig::REMOVE_FIELD_UNDERSCORE_PREFIX, $values[HarmonizeConfig::REMOVE_FIELD_UNDERSCORE_PREFIX])
      ->set(HarmonizeConfig::MAX_RECURSIVE_DEPTH, $values[HarmonizeConfig::MAX_RECURSIVE_DEPTH])
      ->save();

    parent::submitForm($form, $form_state);
  }

}
